"""
:mod:`etlplus` package.

Top-level facade for the ETLPlus toolkit.
"""

from .__version__ import __version__

__author__ = 'ETLPlus Team'


# SECTION: EXPORTS ========================================================== #


__all__ = [
    '__author__',
    '__version__',
]
